--- old/usr/src/uts/common/io/scsi/adapters/mpt_sas3/mptsas3_raid.c	2014-06-12 17:42:17.300395219 +0100
+++ new/usr/src/uts/common/io/scsi/adapters/mpt_sas3/mptsas3_raid.c	2014-06-12 17:42:17.306030121 +0100
@@ -22,6 +22,7 @@
 /*
  * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
+ * Copyright (c) 2014, Tegile Systems Inc. All rights reserved.
  */
 
 /*
@@ -72,20 +73,20 @@
 
 #pragma pack(1)
 
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_type.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_cnfg.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_init.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_ioc.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_raid.h>
-#include <sys/scsi/adapters/mpt_sas/mpi/mpi2_tool.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_type.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_cnfg.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_init.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_ioc.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_raid.h>
+#include <sys/scsi/adapters/mpt_sas3/mpi/mpi2_tool.h>
 
 #pragma pack()
 
 /*
  * private header files.
  */
-#include <sys/scsi/adapters/mpt_sas/mptsas_var.h>
+#include <sys/scsi/adapters/mpt_sas3/mptsas3_var.h>
 
 static int mptsas_get_raid_wwid(mptsas_t *mpt, mptsas_raidvol_t *raidvol);
 
@@ -566,6 +567,7 @@
 void
 mptsas_raid_action_system_shutdown(mptsas_t *mpt)
 {
+	mptsas_reply_pqueue_t		*rpqp;
 	pMpi2RaidActionRequest_t	action;
 	uint8_t				ir_active = FALSE, reply_type;
 	uint8_t				function, found_reply = FALSE;
@@ -573,7 +575,8 @@
 	mptsas_slots_t			*slots = mpt->m_active;
 	int				config, vol;
 	mptsas_cmd_t			*cmd;
-	uint32_t			request_desc_low, reply_addr;
+	uint32_t			reply_addr;
+	uint64_t			request_desc;
 	int				cnt;
 	pMpi2ReplyDescriptorsUnion_t	reply_desc_union;
 	pMPI2DefaultReply_t		reply;
@@ -629,12 +632,13 @@
 
 	/*
 	 * Send RAID Action.
+	 * Defaults to MSIxIndex of 0, so check reply q 0 below.
 	 */
 	(void) ddi_dma_sync(mpt->m_dma_req_frame_hdl, 0, 0,
 	    DDI_DMA_SYNC_FORDEV);
-	request_desc_low = (cmd->cmd_slot << 16) +
+	request_desc = (cmd->cmd_slot << 16) +
 	    MPI2_REQ_DESCRIPT_FLAGS_DEFAULT_TYPE;
-	MPTSAS_START_CMD(mpt, request_desc_low, 0);
+	MPTSAS_START_CMD(mpt, request_desc);
 
 	/*
 	 * Even though reply does not matter because the system is shutting
@@ -642,6 +646,7 @@
 	 * we don't want to leave it hanging if it's coming.  Poll because
 	 * interrupts are disabled when this function is called.
 	 */
+	rpqp = mpt->m_rep_post_queues;
 	for (cnt = 0; cnt < 5000; cnt++) {
 		/*
 		 * Check for a reply.
@@ -650,7 +655,7 @@
 		    DDI_DMA_SYNC_FORCPU);
 
 		reply_desc_union = (pMpi2ReplyDescriptorsUnion_t)
-		    MPTSAS_GET_NEXT_REPLY(mpt, mpt->m_post_index);
+		    MPTSAS_GET_NEXT_REPLY(rpqp, rpqp->rpq_index);
 
 		if (ddi_get32(mpt->m_acc_post_queue_hdl,
 		    &reply_desc_union->Words.Low) == 0xFFFFFFFF ||
@@ -725,20 +730,21 @@
 		 * Clear the reply descriptor for re-use and increment index.
 		 */
 		ddi_put64(mpt->m_acc_post_queue_hdl,
-		    &((uint64_t *)(void *)mpt->m_post_queue)[mpt->m_post_index],
+		    &((uint64_t *)(void *)rpqp->rpq_queue)[rpqp->rpq_index],
 		    0xFFFFFFFFFFFFFFFF);
 		(void) ddi_dma_sync(mpt->m_dma_post_queue_hdl, 0, 0,
 		    DDI_DMA_SYNC_FORDEV);
 
 		/*
-		 * Update the global reply index and keep looking for the
+		 * Update the reply index and keep looking for the
 		 * reply if not found yet.
 		 */
-		if (++mpt->m_post_index == mpt->m_post_queue_depth) {
-			mpt->m_post_index = 0;
+		if (++rpqp->rpq_index == mpt->m_post_queue_depth) {
+			rpqp->rpq_index = 0;
 		}
+
 		ddi_put32(mpt->m_datap, &mpt->m_reg->ReplyPostHostIndex,
-		    mpt->m_post_index);
+		    rpqp->rpq_index);
 		if (!found_reply) {
 			continue;
 		}
